#--- Sample power related functions ---#

#' Estimate detectable effect sizes using statistical power analysis
#'
#' \code{estimateEffectSizes} calculates required sample sizes for a given set of p values and samples.
#'
#' Returns a data.table of effect sizes for a given sample size. Calculates these for p = 0.01, 0.05, 0.1 & 0.2. Pick out the ones you want.
#'
#' @param mean the estimated mean value to use
#' @param sd the estimated stadnard deviation to use
#' @param samples a list of sample sizes to iterate over
#' @param power power value to use
#'
#' @author Ben Anderson, \email{b.anderson@@soton.ac.uk}
#' @export
#' @import data.table
#' @import reshape2
#' @family Power functions

estimateEffectSizes <- function(mean,sd,samples,power){
  # obtain effect sizes using supplied mean & sd
  sigs <- c(0.01,0.05,0.1,0.2) # force these, can always remove later
  nSigs <- length(sigs)
  nSamps <- length(samples)
  # initialise power results array
  resultsArray <- array(numeric(nSamps*nSigs),
                        dim=c(nSamps,nSigs)
  )
  # loop over significance values
  for (p in 1:nSigs){
    for (s in 1:nSamps){ # loop over the sample sizes
      result <- power.t.test( # pwr.t.test?
        n = samples[s],
        delta = NULL,
        sd = sd,
        sig.level = sigs[p],
        power = power,
        alternative = c("one.sided")
      )
      resultsArray[s,p] <- result$delta/mean # report effect size against sample size
    }
  }
  dt <- data.table::as.data.table(resultsArray) # convert to dt for tidying
  dt <- dt[,
           .(
             sampleN = testSamples,
             "p = 0.01" = 100*V1, # "Detectable % effect (p = 0.01)"
             "p = 0.05" = 100*V2, # "Detectable % effect (p = 0.05)"
             "p = 0.1" = 100*V3, # "Detectable % effect (p = 0.1)"
             "p = 0.2" = 100*V4 # "Detectable % effect (p = 0.2)"
           )
           ]
  
  longDT <- data.table::as.data.table(reshape2::melt(dt, id=c("sampleN")))
  longDT <- data.table::setnames(longDT, "value", "effectSize")
  longDT <- data.table::setnames(longDT, "variable", "pValue")
  return(longDT) # returned the tidied & long form dt
}

#' Estimate proportion margins of error using statistical power analysis
#'
#' \code{estimateProportions} calculates required sample sizes for a given set of p values and samples.
#'
#' Returns a data.table of effect sizes for a given sample size. Calculates these for p = 0.01, 0.05, 0.1 & 0.2. Pick out the ones you want.
#'
#' @param mean the estimated mean value to use
#' @param sd the estimated stadnard deviation to use
#' @param samples a list of sample sizes to iterate over
#' @param power power value to use
#'
#' @author Ben Anderson, \email{b.anderson@@soton.ac.uk}
#' @export
#' @import data.table
#' @import reshape2
#' @import pwr
#' @family Power functions

estimateEffectSizes <- function(mean,sd,samples,power){
  # obtain effect sizes using supplied mean & sd
  sigs <- c(0.01,0.05,0.1,0.2) # force these, can always remove later
  nSigs <- length(sigs)
  nSamps <- length(samples)
  # initialise power results array
  resultsArray <- array(numeric(nSamps*nSigs),
                        dim=c(nSamps,nSigs)
  )
  # loop over significance values
  for (p in 1:nSigs){
    for (s in 1:nSamps){ # loop over the sample sizes
      # pwr.t.test?
      result <- power.t.test(
        n = samples[s],
        delta = NULL,
        sd = sd,
        sig.level = sigs[p],
        power = power,
        alternative = c("one.sided")
      )
      resultsArray[s,p] <- result$delta/mean # report effect size against sample size
    }
  }
  dt <- data.table::as.data.table(resultsArray) # convert to dt for tidying
  dt <- dt[,
           .(
             sampleN = testSamples,
             "p = 0.01" = 100*V1, # "Detectable % effect (p = 0.01)"
             "p = 0.05" = 100*V2, # "Detectable % effect (p = 0.05)"
             "p = 0.1" = 100*V3, # "Detectable % effect (p = 0.1)"
             "p = 0.2" = 100*V4 # "Detectable % effect (p = 0.2)"
           )
           ]
  
  longDT <- data.table::as.data.table(reshape2::melt(dt, id=c("sampleN")))
  longDT <- data.table::setnames(longDT, "value", "effectSize")
  longDT <- data.table::setnames(longDT, "variable", "pValue")
  return(longDT) # returned the tidied & long form dt
}


